/************************************************************************

   SgObject.cpp - base class for scene objects

   Written by Petr Mastera (xmaste00).


   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.

************************************************************************/


#include "SgObject.h"

SgObject::SgObject()
{
	Manager=NULL;
	TypeBB=AABB;			
	TypeCollision=BODY;	
	Type=OTHER;				
	Reference=false;					
	Static=false;					
	ID.resize(0);
	SphereRadius=1.0f;				
	ActionRadius=1.0f;				
	for (int i=0;i<3;i++)
	{
		FieldAABB[i]=-1.0f;				
		FieldAAAB[i]=-1.0f;				
	}
	for (int i=3;i<6;i++)
	{
		FieldAABB[i]=1.0f;				
		FieldAAAB[i]=1.0f;				
	}
	selectLight=false;
	Mass=10.0f;	
	CoefficientOfRestitution=1.0f;
	Position=SbVec3f(0,0,0);
	Velocity=SbVec3f(0,0,0);
	AngularVelocity=SbVec3f(0,0,0);
	AngularMomentum=SbVec3f(0,0,0);
	ConstForce=SbVec3f(0,0,0);
	ConstTorque=SbVec3f(0,0,0);
	Damp=0.f;
	InertiaTensor.setValue(SbMatrix(100.f,0.f,0.f,0,
									0.f,100.f,0.0f,0,
									0.f,0,100.f,0,
									0,0,0,1));

/*        Body.OneOverMass = r(1)/Mass;

        Body.InverseBodyInertiaTensor(0,0) = r(3)/(Mass*(dY2*dY2+dZ2*dZ2));
        Body.InverseBodyInertiaTensor(1,1) = r(3)/(Mass*(dX2*dX2+dZ2*dZ2));
        Body.InverseBodyInertiaTensor(2,2) = r(3)/(Mass*(dX2*dX2+dY2*dY2));*/

	createHiddenScene();
}
SgObject::SgObject(const char * model, const char * collisionModel,const SbVec3f & pos, 
				   const SbVec3f & speed, float radius, float mass)
{
	Manager=NULL;
	TypeBB=AABB;			
	TypeCollision=BODY;	
	Type=OTHER;				
	Reference=false;					
	Static=false;					
	ID.resize(0);
	SphereRadius=radius;				
	ActionRadius=radius;				
	for (int i=0;i<3;i++)
	{
		FieldAABB[i]=-1.0f;				
		FieldAAAB[i]=-1.0f;				
	}
	for (int i=3;i<6;i++)
	{
		FieldAABB[i]=1.0f;				
		FieldAAAB[i]=1.0f;				
	}
	selectLight=false;
	Mass=mass;	
	CoefficientOfRestitution=1.0f;
	Position=pos;
	Velocity=speed;
	AngularVelocity=SbVec3f(0,0,0);
	AngularMomentum=SbVec3f(0,0,0);
	ConstForce=SbVec3f(0,0,0);
	ConstTorque=SbVec3f(0,0,0);
	Damp=0.f;
	InertiaTensor.setValue(SbMatrix(1,0,0,0,
									0,1,0,0,
									0,0,1,0,
									0,0,0,1));
	createHiddenScene();
	setModel(model,RENDER);
	setModel(collisionModel,COLLISION);
}
SgObject::~SgObject()
{
  if (Manager) Manager->removeSgo(this);
  releaseHiddenScene();
}
void SgObject::createHiddenScene()
{
  Root = new SoSeparator;
  Root->ref();
  MatrixTransform = new SoMatrixTransform;
  Root->addChild(MatrixTransform);
  Model = NULL;
  CollisionModel = NULL;

  SbMatrix m(MatrixTransform->matrix.getValue());
  m.getTransform(this->Position,this->Orientation,this->Scale,this->ScaleOrientation);
}
void SgObject::releaseHiddenScene()
{
  Root->unref();
  if (CollisionModel) CollisionModel->unref();
#ifndef NDEBUG // debugging purposes only
  Root = NULL;
  MatrixTransform = NULL;
  Model = NULL;
  CollisionModel = NULL;
#endif
}
void SgObject::setSphereRadius(float radius) 
{ 
	this->SphereRadius=radius; 
}
void SgObject::setActionRadius(float radius) 
{ 
	this->ActionRadius=radius; 
}
void SgObject::setFieldAABB(float a0, float a1, float a2, float a3, float a4, float a5)
{
	this->FieldAABB[0]=a0;
	this->FieldAABB[1]=a1;
	this->FieldAABB[2]=a2;
	this->FieldAABB[3]=a3;
	this->FieldAABB[4]=a4;
	this->FieldAABB[5]=a5;
};
void SgObject::setFieldAAAB(float a0, float a1, float a2, float a3, float a4, float a5)
{
	this->FieldAAAB[0]=a0;
	this->FieldAAAB[1]=a1;
	this->FieldAAAB[2]=a2;
	this->FieldAAAB[3]=a3;
	this->FieldAAAB[4]=a4;
	this->FieldAAAB[5]=a5;
};
void SgObject::updateMatrixTransform(){
  SbMatrix m;
  m.setTransform(this->Position, this->Orientation, this->Scale, this->ScaleOrientation);
  this->MatrixTransform->matrix.setValue(m);
}
void SgObject::loadInfo(SoNode * model)
{
	SoSearchAction * search = new SoSearchAction;
	SoVRMLWorldInfo * info;
	string title;

	search->setType(SoVRMLWorldInfo::getClassTypeId());
	search->apply(model);
	if (search->isFound())
	{
		info = (SoVRMLWorldInfo *)search->getPath()->getTail();
		title =((info->title).getValue()).getString();
		if (title.compare("IT&BB")==0)
		{
			int count = (info->info).getNum();
			if (count==16)
			{
				string pom;
				float m[9];
				for (int i=0;i<9;i++) 
				{
					pom = (info->info)[i].getString();
					//m[i]= (this->getScale()[0]) * (float)(atof(pom.c_str()));
					m[i]= (float)(atof(pom.c_str()));
				}
				this->InertiaTensor.setValue(SbMatrix(m[0],m[1],m[2],0,
												m[3],m[4],m[5],0,
												m[6],m[7],m[8],0,
												0,0,0,1));
				massOverIT();
				
				for (int i=9;i<15;i++)

				{
					pom = (info->info)[i].getString();
					m[i-9] = (this->getScale()[0]) * (float)(atof(pom.c_str()));
				}
				this->setFieldAABB(m[0],m[1],m[2],m[3],m[4],m[5]);
				this->setFieldAAAB(m[0],m[1],m[2],m[3],m[4],m[5]);
				pom = (info->info)[15].getString();
				this->setSphereRadius((this->getScale()[0]) * (float)(atof(pom.c_str())));
				this->setActionRadius((this->getScale()[0]) * (float)(atof(pom.c_str())));
			}
		}
	}
}
void SgObject::massOverIT()
{
	SbMatrix m(this->InertiaTensor);
	float mass=this->Mass;
	SbVec4f r0,r1,r2,r3;
	r0=m[0];
	r1=m[1];
	r2=m[2];
	r3=m[3];
	r0 = SbVec4f((r0[0]!=0)?mass/r0[0]:0,(r0[1]!=0)?mass/r0[1]:0,(r0[2]!=0)?mass/r0[2]:0,(r0[3]!=0)?mass/r0[3]:0);
	r1 = SbVec4f((r1[0]!=0)?mass/r1[0]:0,(r1[1]!=0)?mass/r1[1]:0,(r1[2]!=0)?mass/r1[2]:0,(r1[3]!=0)?mass/r1[3]:0);
	r2 = SbVec4f((r2[0]!=0)?mass/r2[0]:0,(r2[1]!=0)?mass/r2[1]:0,(r2[2]!=0)?mass/r2[2]:0,(r2[3]!=0)?mass/r2[3]:0);
	r3 = SbVec4f((r3[0]!=0)?mass/r3[0]:0,(r3[1]!=0)?mass/r3[1]:0,(r3[2]!=0)?mass/r3[2]:0,(r3[3]!=0)?mass/r3[3]:0);
	m.setValue(SbMatrix(r0[0],r0[1],r0[2],r0[3],
						r1[0],r1[1],r1[2],r1[3],
						r2[0],r2[1],r2[2],r2[3],
						r3[0],r3[1],r3[2],r3[3]));
	this->InversIT.setValue(m);
	this->ActualInversIT.setValue(m);
}
SbMatrix SgObject::OrthoNormalize(SbMatrix & m)
{
	SbVec4f r0,r1,r2,r3;
	r0=m[0]; r1=m[1]; r2=m[2]; r3=m[3];
	SbVec3f x,y,z;
	x=SbVec3f(r0[0],r1[0],r1[0]);
	y=SbVec3f(r0[1],r1[1],r2[1]);
	if (x.length()>0) x.normalize();
	z=x.cross(y);
	if (z.length()>0) z.normalize();
	y=x.cross(z);
	if (y.length()>0) y.normalize();
	SbMatrix pom(	x[0],y[0],z[0],0,
					x[1],y[1],z[1],0,
					x[2],y[2],z[2],0,
					0   ,0   ,0   ,1);
    return pom;
}
void SgObject::setModel(SoNode * model, MODEL_SET which, bool info)
{
	if (info) this->loadInfo(model);
	if ((which & RENDER) != 0) {
		if (this->Model) Root->removeChild(this->Model);
		this->Model = model;
		if (model) Root->addChild(model);
	}
	if ((which & COLLISION) != 0) {
		if (this->CollisionModel) this->CollisionModel->unref();
		this->CollisionModel = model;
		if (this->CollisionModel) this->CollisionModel->ref();
	}
}
SbBool SgObject::setModel(const char *filename, MODEL_SET which, bool info)
{
  if (!which) return TRUE;
  SoInput in;
  if (!in.openFile(filename)) return FALSE;
  SoSeparator * model = SoDB::readAll(&in);
  model->ref();
  setModel(model, which, info);
  model->unref();
  return TRUE;
}

SbVec3f SgObject::getDirVel() const
{
	SbVec3f dirvel=this->Velocity;
	if (dirvel.length()>0)
	{
		dirvel.normalize();
		return dirvel;
	}else
	{
		SbVec3f dirvec(0, 0, 1);
		this->CameraOrientation.multVec(dirvel,dirvel);
		if (dirvel.length()>0) dirvel.normalize();
		else dirvel=SbVec3f(0, 0, 1);
		return dirvel;
	}
}
SbVec3f SgObject::getDirVec()
{
	SbVec3f dirvec(0, 0, 1);
	//this->Orientation.multVec(dirvec,dirvec);
	this->CameraOrientation.multVec(dirvec,dirvec);
	if (dirvec.length()>0) dirvec.normalize();
	else dirvec=SbVec3f(0, 0, 1);
	return dirvec;
}
SbVec3f SgObject::getUpVec()
{
	SbVec3f upvec(0, 1, 0);
	//this->Orientation.multVec(upvec,upvec);
	this->CameraOrientation.multVec(upvec,upvec);
	if (upvec.length()>0) upvec.normalize();
	else upvec=SbVec3f(0, 0, 1);
	return upvec;
}
SbVec3f SgObject::getLeftVec()
{
	SbVec3f leftvec(1, 0, 0);
	//this->Orientation.multVec(leftvec,leftvec);
	this->CameraOrientation.multVec(leftvec,leftvec);
	if (leftvec.length()>0) leftvec.normalize();
	else leftvec=SbVec3f(0, 0, 1);
	return leftvec;
}
void SgObject::setOrientation(const SbRotation &r) 
{ 
	this->Orientation=r; 
	this->CameraOrientation=this->Orientation;
	if (this->Type==this->FIGHTER)
	{
		this->CameraOrientation*=SbRotation(this->getUpVec(),(float)(-0.5f*M_PI));
		this->CameraOrientation*=SbRotation(this->getDirVec(),(float)(0.5f*M_PI));
	}
}
void SgObject::setOrientation(const SbVec3f &axis, const float radians) 
{ 
	this->Orientation=SbRotation(axis,radians); 
	this->CameraOrientation=this->Orientation;
	if (this->Type==this->FIGHTER)
	{
		this->CameraOrientation*=SbRotation(this->getUpVec(),(float)(-0.5f*M_PI));
		this->CameraOrientation*=SbRotation(this->getDirVec(),(float)(0.5f*M_PI));
	}
}
void SgObject::updOrientation(const SbVec3f &axis, const float radians) 
{ 
	this->Orientation*=SbRotation(axis,radians); 
	this->CameraOrientation=this->Orientation;
	if (this->Type==this->FIGHTER)
	{
		this->CameraOrientation*=SbRotation(this->getUpVec(),(float)(-0.5f*M_PI));
		this->CameraOrientation*=SbRotation(this->getDirVec(),(float)(0.5f*M_PI));		
	}
}
void SgObject::RotLR(const float radians) 
{ 
	this->Orientation*=SbRotation(this->getUpVec(),radians); 
	this->CameraOrientation=this->Orientation; 
	if (this->Type==this->FIGHTER)
	{
		this->CameraOrientation*=SbRotation(this->getUpVec(),(float)(-0.5f*M_PI));
		this->CameraOrientation*=SbRotation(this->getDirVec(),(float)(0.5f*M_PI));		
	}
}		
void SgObject::RotUD(const float radians) 
{ 
	this->Orientation*=SbRotation(this->getLeftVec(),radians); 
	this->CameraOrientation=this->Orientation; 
	if (this->Type==this->FIGHTER)
	{
		this->CameraOrientation*=SbRotation(this->getUpVec(),(float)(-0.5f*M_PI));
		this->CameraOrientation*=SbRotation(this->getDirVec(),(float)(0.5f*M_PI));		
	}
}
void SgObject::RotAR(const float radians) 
{ 
	this->Orientation*=SbRotation(this->getDirVec(),radians); 
	this->CameraOrientation=this->Orientation; 
	if (this->Type==this->FIGHTER)
	{
		this->CameraOrientation*=SbRotation(this->getUpVec(),(float)(-0.5f*M_PI));
		this->CameraOrientation*=SbRotation(this->getDirVec(),(float)(0.5f*M_PI));		
	}
}
void SgObject::TorqueLR(const float torque)
{ 
	this->AngularVelocity+=this->getLeftVec().cross(this->getDirVec()*torque); 
}
void SgObject::TorqueUD(const float torque) 
{ 
	this->AngularVelocity+=this->getDirVec().cross(this->getUpVec()*torque); 
}
void SgObject::TorqueAR(const float torque) 
{ 
	this->AngularVelocity+=this->getLeftVec().cross(this->getUpVec()*torque); 
}

void SgObject::timeStep(double time, double dt)
{
	//if ((!this->Static)&&(this->Type==SgObject::FIGHTER))
	if (!this->Static)
	{
		for (int i=0; i<Light.size(); i++)
		{
			this->Light[i]->setPos(this->Position);		// aktualizuji pozici svetla
			this->Light[i]->setDir(this->getDirVec());	// aktualizuji smer svetla
		}
		this->OldPosition = this->Position;
		this->OldOrientation = this->Orientation;
		
		//this->Damp=0.5f;
		// integrace sil
		this->Force=this->ConstForce;
		this->Force+= -Damp * this->Velocity;
		this->Torque=this->ConstTorque;
		this->Torque+= -Damp * this->AngularVelocity;

		// integrace translace
		this->updPos(this->Velocity * float(dt));
		// integrace rotace
 		float size=this->AngularVelocity.length();
		if (size) this->updOrientation(this->AngularVelocity,(size)*float(dt));

		this->Velocity+=this->Force * float (dt);
		this->AngularVelocity+=this->Torque * float (dt);

		SbMatrix m1,mt;
		this->Orientation.getValue(m1);
		mt=m1.transpose();
		//this->ActualInversIT=m1*this->ActualInversIT*mt;
		this->ActualInversIT=m1*this->InversIT*mt;
		this->ActualInversIT.multMatrixVec(this->AngularVelocity,this->AngularMomentum);

//
//
//
//		//kamera neni ovlivnena silou
//
//		if ((Damp>0) && (this->Velocity.length()>0))
//		{
//			if (this->Velocity.length()>0.01) this->Force=-this->getDirVel()*this->Damp;
//			else
//			{
//				this->Velocity=SbVec3f(0,0,0);
//				this->Force=SbVec3f(0,0,0);
//			}
//		}
//		if (this->Type!=SgObject::CAMERA)	
//			this->Velocity+=this->Force * float(dt);
//		
//		// integrace pozice
//		
//		// integrace rotace
//		
//
//		if ((Damp>0)&&(this->AngularMomentum.length()>0)) 
//		{
//			if (this->AngularVelocity.length()>0.1) this->Torque=-this->AngularMomentum*this->Damp;
//			else 
//			{
//				this->AngularMomentum=SbVec3f(0,0,0);
//				this->Torque=SbVec3f(0,0,0);
//			}
//		}
//		this->AngularMomentum+=this->Torque * float(dt);
//		
//		float size=this->AngularVelocity.length();
//		if (size) 
//		{
//			if (Damp>0) size-=Damp*float(dt);
//			this->updOrientation(this->AngularVelocity,(size)*float(dt));
//		}
//		
//		
//		SbMatrix m1,mt;
//		float k=float(dt);
//		
////		if (size) this->updOrientation(this->AngularMomentum,-(size)*float(dt)/180.f);
//
///*   Target.Orientation = Source.Orientation +
//                DeltaTime *
//                matrix_3x3(Source.AngularVelocity,matrix_3x3::SkewSymmetric) *
//                Source.Orientation;*/
//		
///*		float size=this->AngularVelocity.length();
//		if (size){
//			//cout<<"axis    "<<this->AngularMomentum[0]<<" "<<this->AngularMomentum[1]<<" "<<this->AngularMomentum[2]<<endl;
//			cout<<"size dt "<<size*(float)(dt)<<endl;
//			//this->setOrientation(this->AngularVelocity,(size)*float(dt)/180.f);
//			this->updOrientation(this->AngularVelocity,(size)*float(dt)/180.f);
//		}*/
//
//
//		this->Orientation.getValue(m1);
//		//m1=this->OrthoNormalize(m1);
//		//this->Orientation.setValue(m1);
//
//
//
//		SbMatrix v;
//		SbVec4f r0,r1,r2,r3;
//		v=this->ActualInversIT;
//		r0=v[0];
//		r1=v[1];
//		r2=v[2];
//		r3=v[3];
//		//cout<<"Tensor"<<endl;
//		//cout<<"  "<<r0[0]<<"  "<<r0[1]<<"  "<<r0[2]<<"  "<<r0[3]<<endl;
//		//cout<<"  "<<r1[0]<<"  "<<r1[1]<<"  "<<r1[2]<<"  "<<r1[3]<<endl;
//		//cout<<"  "<<r2[0]<<"  "<<r2[1]<<"  "<<r2[2]<<"  "<<r2[3]<<endl;
//		//cout<<"  "<<r3[0]<<"  "<<r3[1]<<"  "<<r3[2]<<"  "<<r3[3]<<endl;
//		//cout<<v.det3()<<endl;
//		
//		
//		mt=m1.transpose();
//		//InversIT=this->OrthoNormalize(this->InversIT);
//		
//		this->ActualInversIT=m1*this->InversIT*mt;
//		this->ActualInversIT.multMatrixVec(this->AngularMomentum,this->AngularVelocity);
//		//this->ActualInversIT=m1*this->ActualInversIT*mt;
//
//
//
////cout<<"AngMom "<<AngularMomentum[0]<<" "<<AngularMomentum[1]<<" "<<AngularMomentum[2]<<endl;
////cout<<"AngVel "<<AngularVelocity[0]<<" "<<AngularVelocity[1]<<" "<<AngularVelocity[2]<<endl;
//		this->ActualInversIT.multMatrixVec(this->AngularMomentum,this->AngularVelocity);
////cout<<"AngMom "<<AngularMomentum[0]<<" "<<AngularMomentum[1]<<" "<<AngularMomentum[2]<<endl;
////cout<<"AngVel "<<AngularVelocity[0]<<" "<<AngularVelocity[1]<<" "<<AngularVelocity[2]<<endl;
//		
//		//cout<<"vel    "<<this->AngularVelocity[0]<<" "<<this->AngularVelocity[1]<<" "<<this->AngularVelocity[2]<<endl;
//
//		//v=this->ActualInversIT;
//		//r0=v[0];
//		//r1=v[1];
//		//r2=v[2];
//		//r3=v[3];
//		//cout<<"Tensor"<<endl;
//		//cout<<"  "<<r0[0]<<"  "<<r0[1]<<"  "<<r0[2]<<"  "<<r0[3]<<endl;
//		//cout<<"  "<<r1[0]<<"  "<<r1[1]<<"  "<<r1[2]<<"  "<<r1[3]<<endl;
//		//cout<<"  "<<r2[0]<<"  "<<r2[1]<<"  "<<r2[2]<<"  "<<r2[3]<<endl;
//		//cout<<"  "<<r3[0]<<"  "<<r3[1]<<"  "<<r3[2]<<"  "<<r3[3]<<endl;
//		//cout<<"Determinant = "<<v.det3()<<endl;
//		//cout<<endl;
	}
	this->updateMatrixTransform();

	//cout<<this->Position[0]<<" "<<this->Position[1]<<" "<<this->Position[2]<<endl;


    /*Target.InverseWorldInertiaTensor = Target.Orientation *
                aBodies[Counter].InverseBodyInertiaTensor *
                Transpose(Target.Orientation);
*/

}