#ifndef SG_OBJECT_H_ZERO
#define SG_OBJECT_H_ZERO
/************************************************************************

   SgObject.h - base class for scene objects

   Written by Petr Mastera (xmaste00).


   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.

************************************************************************/

#include "MainInclude.h"
#include "SgInterface.h"
#include "SgoManager.h"

class SoNode;
class SoSeparator;
class SoMatrixTransform;
class SgoManager;
class SbVec3f;
class SgLight;

class SgObject {
public:
	enum MODEL_SET { RENDER = 0x01, COLLISION = 0x02, ALL = 0x03 };
	enum TYPE_OBJECT_BB { SPHERE = 0x01, AABB = 0x02, OBB = 0x03, KDOP = 0x04 };
	enum TYPE_COLLISION { BODY = 0x01, BALL = 0x02 };
	enum TYPE_OBJECT { OBJECT = 0x01, FIGHTER = 0x02, MISSILE = 0x03, CAMERA = 0x04, OTHER = 0x05 };
protected:
	// OpenInventor Variables
	SgoManager * Manager;				// sprava objektu
	SoSeparator * Root;					// koren objektu
	SoMatrixTransform * MatrixTransform;// transformacnimatice objektu

	// Model Variables
	SoNode * Model;						// model objektu
	SoNode * CollisionModel;			// kolizni model objektu
	SbVec3f Scale;						// for MatrixTanfosrm

	// User Variables
	TYPE_OBJECT_BB TypeBB;			// typ BoundingBoxu
	TYPE_COLLISION TypeCollision;	// typ jak objekt koliduje
	TYPE_OBJECT Type;				// typ objektu
	bool Reference;					// zda definuje vlastni podprostor
	bool Static;					// zda je staticky objekt
	vector<int> ID;					// identifikace reference/podprostoru
	float SphereRadius;				// polomer ohranicujici koule telesa
	float ActionRadius;				// polomer akcniho radiu koule telesa
	float FieldAABB[6];				// rozmery osove orientovaneho ohranicujiciho kvadru telesa ([-x,-y,-z],[x,y,z])
	float FieldAAAB[6];				// rozmery osove orientovaneho akcniho kvadru telesa ([-x,-y,-z],[x,y,z])

	vector<SgLight*>  Light;		// ukazatel na pouzite svetla
	bool selectLight;				// zda je na objektu svetlo

/*	// Move
	int durFB;
	int durLR;
	int durUD;
	int durX;
	int durY;
	int durZ;*/

	// Physical Variables
	float Mass;						// hmotnost objektu
	float CoefficientOfRestitution;	// soucinitel odrayivosti
	SbVec3f Position;				// pozice objektu // for MatrixTanfosrm
	SbVec3f OldPosition;			// stara pozice
	SbVec3f Velocity;				// rychlost
	SbVec3f AngularVelocity;		// uhlova rychlost
	SbVec3f AngularMomentum;		// uhlovy moment
	SbVec3f Force;					// translacni sili pusobici na teleso
	SbVec3f Torque;					// rotacni sili pusobici na teleso
	SbVec3f ConstForce;				// konstantni translacni sili pusobici na teleso
	SbVec3f ConstTorque;			// konstantni rotacni sili pusobici na teleso
	float Damp;						// tlumeni (odpor prostredi)
	SbRotation Orientation;			// orientace telesa // for MatrixTanfosrm
	SbRotation OldOrientation;		// stara orientace telesa
	SbRotation CameraOrientation;	// orientace kamery
	SbRotation ScaleOrientation;	// for MatrixTanfosrm
	SbMatrix InertiaTensor;			// moment setrvacnosti
	SbMatrix InversIT;				// inverzni moment setrvacnosti
	SbMatrix ActualInversIT;		// moment setrvacnosti podle aktualniho natoceni

	// Metods
	void createHiddenScene();		// vytvori pocatecni scenu a inicializuje root
	void releaseHiddenScene();		// uvolni scenu
	void updateMatrixTransform();	// aktualizuje transformacni matici

	void loadInfo(SoNode * model);	// nahraje doplnujici informace o modelu
	void massOverIT();				// mass/IT
	SbMatrix OrthoNormalize(SbMatrix & m);	// provede ortho normalizaci matice

public:
	SgObject();
	SgObject(const char *model, const char *collisionModel = NULL,
			const SbVec3f &pos = SbVec3f(0.f,0.f,0.f), const SbVec3f &dir = SbVec3f(0.f,0.f,1.f),
			float radius = 0.f, float weight = 1.f);
	virtual ~SgObject();
	// OpenInventor Variables
	SgoManager * getManager() const { return Manager; }
	void setManager(SgoManager * manager) { Manager=manager; }
	SoSeparator * getSceneGraph() const { return Root; }
	SoMatrixTransform * getMatrixTransform() { return MatrixTransform; }
	const SbMatrix & getMatrix() const { return MatrixTransform->matrix.getValue(); }
	void setMatrix(const SbMatrix & m) { MatrixTransform->matrix=m; }
	// Model Variables
	SoNode * getModel() const { return Model; }
	SoNode * getCollisionModel() const { return CollisionModel; }
	void setModel(SoNode * model, MODEL_SET which = ALL, bool info = true);
	SbBool setModel(const char * filename, MODEL_SET which = ALL, bool info = true);
	void setScale(SbVec3f scale) { Scale=scale; }
	SbVec3f getScale() const { return Scale; }
	// User Variables
	void setTypeBB(TYPE_OBJECT_BB t) { this->TypeBB=t; }
	TYPE_OBJECT_BB getTypeBB() { return this->TypeBB; }
	void setTypeCollision (TYPE_COLLISION t) { this->TypeCollision=t; }
	TYPE_COLLISION getTypeCollision () { return this->TypeCollision; }
	void setType (TYPE_OBJECT t) { this->Type=t; }
	TYPE_OBJECT getType () { return this->Type; }
	void setReference(bool b) { this->Reference=b; }
	bool getReference() { return this->Reference; }
	void setStatic(bool b) { this->Static=b; }
	bool getStatic() { return this->Static; }

	void clearID() { this->ID.resize(1); this->ID[0]=-1; }
	void addID(int id) { this->ID.push_back(id); }
	void setID(int id) { this->ID.resize(1); this->ID[0]=id;}
	int getNumID() { return (int)this->ID.size(); }
	int getID(int index) {return this->ID[index]; }

	void setSphereRadius(float radius);
	float getSphereRadius() const { return this->SphereRadius; }
	void setActionRadius(float radius);
	float getActionRadius() const { return this->ActionRadius; }
	void setFieldAABB(float a0, float a1, float a2, float a3, float a4, float a5);
	float getOneAABB(int index) { return this->FieldAABB[index]; }
	void setFieldAAAB(float a0, float a1, float a2, float a3, float a4, float a5);
	float getOneAAAB(int index) { return this->FieldAAAB[index]; }
  
	void addLigth(SgLight * light) { this->Light.push_back(light) ; this->selectLight=true; }
	//void remLight(int index)
/*	// move
	void incDurFB() { durFB++; }
	void incDurLR() { durLR++; }
	void incDurUD() { durUD++; }
	void incDurX() { durX++; }
	void incDurY() { durY++; }
	void incDurZ() { durZ++; }
	void decDurFB() { durFB--; }
	void decDurLR() { durLR--; }
	void decDurUD() { durUD--; }
	void decDurX() { durX--; }
	void decDurY() { durY--; }
	void decDurZ() { durZ--; }
	bool zeroDurFB() { if (durFB) return false; else return true; }
	bool zeroDurLR() { if (durLR) return false; else return true; }
	bool zeroDurUD() { if (durUD) return false; else return true; }
	bool zeroDurX() { if (durX) return false; else return true; }
	bool zeroDurY() { if (durY) return false; else return true; }
	bool zeroDurZ() { if (durZ) return false; else return true; }*/

	// Physical Variables
	void setMass(float mass) { Mass=mass; this->massOverIT(); }
	float getMass() const { return Mass; }
	void setCoefficientOfRestitution(float e) { CoefficientOfRestitution=e; }
	float getCoefficientOfRestitution() const { return CoefficientOfRestitution; }
	
	void setForce(SbVec3f f) { this->ConstForce=f; }
	SbVec3f getForce() { return this->ConstForce; }
	void setTorque(SbVec3f f) { this->ConstTorque=f; }
	SbVec3f getTorque() { return this->ConstTorque; }
	void setDamp(float d) { this->Damp=d; }
	float getDamp() { return this->Damp; }

	// position
	SbVec3f getDirVel()const;
	SbVec3f getDirVec();
	SbVec3f getUpVec();
	SbVec3f getLeftVec();
	SbVec3f& getPos() { return this->Position; }
	void setPos(const SbVec3f &p) { this->Position=p; }
	void setPos(const float x, const float y, const float z) { this->Position=SbVec3f(x,y,z); }
	void updPos(const SbVec3f &deltaP) { this->Position+=deltaP; }
	void updPos(const float x, const float y, const float z) { this->Position+=SbVec3f(x,y,z); }
	SbVec3f& getOldPos() { return this->OldPosition; }
	void setOldPos() { this->Position=this->OldPosition; }

	// velocity
	SbVec3f getVel() { return this->Velocity; }
	void setVel(const SbVec3f &v) { this->Velocity=v; }
	void setVel(const float x, const float y, const float z) { this->Velocity=SbVec3f(x,y,z); }
	SbVec3f getAngVel() { return this->AngularVelocity; }
	void setAngVel(const SbVec3f &av) { this->AngularVelocity=av; }
	void setAngVel(const float x, const float y, const float z) { this->AngularVelocity=SbVec3f(x,y,z); }
	SbVec3f getAngMom() { return this->AngularMomentum; }
	void setAngMom(const SbVec3f &am) { this->AngularMomentum=am; }
	void setAngMom(const float x, const float y, const float z) { this->AngularMomentum=SbVec3f(x,y,z); }

	// rotation
	SbRotation getOrientation() { return this->Orientation; }
	SbRotation getCameraOrientation() { return this->CameraOrientation; }
	void setOrientation(const SbRotation &r);// { this->Orientation=r; }
	void setOrientation(const SbVec3f &axis, const float radians);// { this->Orientation=SbRotation(axis,radians); }
	void updOrientation(const SbVec3f &axis, const float radians);// { this->Orientation*=SbRotation(axis,radians); }
	void RotLR(const float radians);		//LeftRight //otaceni vlevo vpravo
	void RotUD(const float radians);		//UpDown	//otaceni nahoru dolu
	void RotAR(const float radians);		//all round	//otaceni kolem dokola v ose pohybu
	void TorqueLR(const float torque);		//Left/Rignht	//otaceni vlevo vpravo
	void TorqueUD(const float torque);		//otaceni nahoru dolu
	void TorqueAR(const float torque);		//all round		//otaceni kolem dokola v ose pohybu
	//void TorqueLR(const float torque) { this->AngularMomentum+=this->getLeftVec().cross(this->getDirVec()*torque); }//Left/Rignht	//otaceni vlevo vpravo
	//void TorqueUD(const float torque) { this->AngularMomentum+=this->getDirVec().cross(this->getUpVec()*torque); }	//Up/Down		//otaceni nahoru dolu
	//void TorqueAR(const float torque) { this->AngularMomentum+=this->getLeftVec().cross(this->getUpVec()*torque); }	//all round		//otaceni kolem dokola v ose pohybu
	SbRotation getOldOrientation() { return this->OldOrientation; }
	void setOldOrientation() { this->Orientation=this->OldOrientation; }

	// InertiaTensor
	SbMatrix getInertiaTensor() { return this->ActualInversIT; }
	void setInertiaTensor(const SbMatrix &m) { this->InversIT=m; }

	// speed,direction
	const SbVec3f& getSpeed() const  { return this->Velocity; }
	void setSpeed(const SbVec3f &speed) { this->Velocity=speed; }
	void setSpeed(const float x, const float y, const float z) { this->Velocity=SbVec3f(x,y,z); }

	// metod
	virtual void timeStep(double time, double dt);
};

#endif /* SG_OBJECT_H */
