/************************************************************************
   SgoManager.cpp - scene objects manager class

   Written by Petr Mastera.

   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.
************************************************************************/
#include "SgoManager.h"

SgoManager::SgoManager()
{
  sgoRoot = new SoSeparator;
  sgoRoot->ref();

  this->Force=SbVec3f(0,0,0);
  this->Torque=SbVec3f(0,0,0);
  this->Damp=0.f;
  this->CollidingSgos.resize(0);
  this->epsilon=0.000001f;
	
  collisionRoot = new SoGroup;
  collisionRoot->ref();
  collisionRoot->addChild(new SoSeparator);
  collisionRoot->addChild(new SoSeparator);

  ida = new SoIntersectionDetectionAction;
  ida->setManipsEnabled(FALSE);
  ida->setDraggersEnabled(FALSE);
  ida->setIntersectionDetectionEpsilon(0.0f);
  //ida->setFilterCallback(filterCB, NULL);
  ida->addIntersectionCallback(intersectionCb, this);

  fighterDestroy=false;
}
SgoManager::~SgoManager()
{
  sgoRoot->unref();
  collisionRoot->unref();
}
void SgoManager::appendSgo(SgObject *sgo)
{
  if (sgo->getManager()) {
    if (sgo->getManager() == this)  return;
    else  removeSgo(sgo);
  }
  sgoList.append(sgo);
  sgoRoot->addChild(sgo->getSceneGraph());
  sgo->timeStep(0,0);
  sgo->setManager(this);
}
void SgoManager::removeSgo(SgObject *sgo)
{
  int i = sgoList.find(sgo);
  assert(i != -1 && "Object not present.");
  removeSgo(i);
}
void SgoManager::removeSgo(int index)
{
  assert(index >= 0 && index < sgoList.getLength() && "Index out of bounds.");
  SgObject *sgo = sgoList[index];
  SoSeparator *graph = sgo->getSceneGraph();
  assert(graph && "Object has not associated its scene graph.");
  sgoRoot->removeChild(graph);
  sgoList.remove(index);
  sgo->setManager(NULL);
}
void SgoManager::deleteSgo(int index)
{
  assert(index >= 0 && index < sgoList.getLength() && "Index out of bounds.");
  SgObject *sgo = sgoList[index];
  removeSgo(index);
  delete sgo;
}
void SgoManager::deleteSgo(SgObject *sgo)
{
  int i = sgoList.find(sgo);
  assert(i != -1 && "Object not present.");
  deleteSgo(i);
}
void SgoManager::deteleAllSgos()
{
  int i = sgoList.getLength();
  while (--i >= 0)
    deleteSgo(i);
}
void SgoManager::checkCollision(const double time, const double dt)
{
	for (int i=0; i< sgoList.getLength(); i++){			// pres vsechny objekty sceny
		//sgoList[i]->setForce(SbVec3f(0,0,0));			// vynuluju vsechny pusobici sily
		if (!(sgoList[i]->getReference()))				// pro vsechny nereferencni objekty
			sgoList[i]->clearID();						// se odeberou reference na prostory
	}
	this->CollidingSgos.resize(0);						// vynuluju vsechny kolize
	int c = sgoList.getLength();
	for (int i=0; i<c; i++)		// pres vsechny objekty sceny
	{		
		sgoList[i]->setForce(this->Force);
		sgoList[i]->setDamp(this->Damp);
		sgoList[i]->timeStep(time, dt);
		if (i<(c-1))
		{
			this->checkCollisionWith(sgoList[i],i+1);		// hledam kolize s kazdym dalsim objektem (predposledni a posledni)
			this->resolveCollision(time,dt);
			this->CollidingSgos.resize(0);
		}
		//sgoList[i]->timeStep(time, dt);
		c = sgoList.getLength();
	}
}
bool SgoManager::checkID(SgObject * sgo1, SgObject * sgo2)
{
	for (int i=0; i<sgo1->getNumID();i++)
		for (int j=0; j<sgo2->getNumID();j++)
			if (sgo1->getID(i)==sgo2->getID(j)) return true;
	return false;
};
void SgoManager::checkTriangleCollision(int sgo1, int sgo2)
{
	SgObject * so1 = this->sgoList[sgo1];
	SgObject * so2 = this->sgoList[sgo2];
	this->CollidingSgos.resize(CollidingSgos.size()+1);
	this->CollidingSgos.back().Index[0]=sgo1;
	this->CollidingSgos.back().Index[1]=sgo2;
	this->CollidingSgos.back().Type=SgoManager::BODY;
	this->CollidingSgos.back().Collision=false;
	assert(((SoGroup*)collisionRoot->getChild(0))->getNumChildren() == 0);
	assert(((SoGroup*)collisionRoot->getChild(1))->getNumChildren() == 0);
	((SoGroup*)collisionRoot->getChild(0))->addChild(so1->getMatrixTransform());
	((SoGroup*)collisionRoot->getChild(0))->addChild(so1->getCollisionModel());
	((SoGroup*)collisionRoot->getChild(1))->addChild(so2->getMatrixTransform());
	((SoGroup*)collisionRoot->getChild(1))->addChild(so2->getCollisionModel());
	ida->apply(collisionRoot);
	if (!this->CollidingSgos.back().Collision ) 
		this->CollidingSgos.pop_back();
	((SoGroup*)collisionRoot->getChild(0))->removeAllChildren();
	((SoGroup*)collisionRoot->getChild(1))->removeAllChildren();
};
void SgoManager::checkCollisionWith(SgObject * so, int index2)
{
	int index1=index2-1;								// index na prvni zjistovany objekt
	vector<SgObject*> pom;								// pomocne pole pro dva porovnavane objekty
	pom.push_back(so);									// vlozeni pvniho objektu do pomocneho pole
	int in0=0;			// prvni index na pozici
	int in1=1;			// druhy index na pozici
	bool sphereColl=false;
	bool blockColl=false;
	bool onlySphere=false;
	bool onlyBlock=false;
	bool SphereBlock=false;
	bool CollSphere=false;
	bool CollBlock=false;
	bool CollSphereBlock=false;
	bool Camera=false;
	SbVec3f distance(0,0,0);
	double lenght=0;
	int X=0;
	int Y=1;
	int Z=2;
	int min=0;
	int max=3;

	//for (int i=index2; i<sgoList.getLength(); i++)		// pro vsechny dalsi objekty
	for (int i=0; i<sgoList.getLength(); i++)				// pro vsechny dalsi objekty
	if (i!=index1)
	{
		pom.push_back(sgoList[i]);						// vlozeni druheho objektu do pomocneho pole
		if (!(pom[0]->getStatic() && pom[1]->getStatic()))				// pokracuj jen pokud nejsou oba staticke
		{
			Camera=((pom[0]->getType()==SgObject::CAMERA)||(pom[1]->getType()==SgObject::CAMERA));
			onlySphere=((pom[0]->getTypeBB()==SgObject::SPHERE)&&(pom[1]->getTypeBB()==SgObject::SPHERE));	// kolize mezi obalovimi koulemi
			onlyBlock=((pom[0]->getTypeBB()==SgObject::AABB)&&(pom[1]->getTypeBB()==SgObject::AABB));			// kolize mezi obalovimi bloky
			SphereBlock=(((pom[0]->getTypeBB()==SgObject::SPHERE)&&(pom[1]->getTypeBB()==SgObject::AABB))	||	// kolize koule a blok
						((pom[1]->getTypeBB()==SgObject::SPHERE)&&(pom[0]->getTypeBB()==SgObject::AABB)));
			CollSphere=((pom[0]->getTypeCollision()==SgObject::BALL)&&(pom[1]->getTypeCollision()==SgObject::BALL));	// vyhodnoceni odrazu pro koule
			CollBlock=((pom[0]->getTypeCollision()==SgObject::BODY)&&(pom[1]->getTypeCollision()==SgObject::BODY));		// vyhodnoveni odrazu pro pevne telesa
			CollSphereBlock=(((pom[0]->getTypeCollision()==SgObject::BALL)&&(pom[1]->getTypeCollision()==SgObject::BODY)) ||// srazka koule a telesa
						((pom[1]->getTypeCollision()==SgObject::BALL)&&(pom[0]->getTypeCollision()==SgObject::BODY)));

			// rozdeleni prostoru na podprostory podle referenci
			if (pom[0]->getReference() || pom[1]->getReference())		// pokracuj pokud je aspon jeden reference
			{
				if (!(pom[0]->getReference() && pom[1]->getReference()))// pokracuj pokud nejsou oba dva referencni
				{
					if (pom[0]->getReference())							// referencni teleso
					{
						in0=0;											// zaradim na prvni misto
						in1=1;
					} else 
					{
						in0=1;
						in1=0;
					}
					// zjistim typ boundingBoxu
					if (onlySphere)	// koule
					{
						sphereColl=false;
						distance=pom[0]->getPos()-pom[1]->getPos();		// spocitam jejich vyajemnou vydalenost
						lenght=distance[0]*distance[0]+distance[1]*distance[1]+distance[2]*distance[2];
																		// pokud je vetsi nez polomer reference a druhe obalove koule
						if (lenght < ((pom[in0]->getActionRadius()+pom[in1]->getSphereRadius())*
									  (pom[in0]->getActionRadius()+pom[in1]->getSphereRadius()))) 
									  sphereColl=true;					// teleso se nachazi v podprostoru reference
						if (sphereColl&&!Camera)						// kdyz se nachazi v podprostoru
							if (pom[in0]->getNumID()>0)					// za predpokladu ze reference ma nejake ID
								pom[in1]->addID(pom[in0]->getID(0));	// priradim stejne ID do telesa
					}
					if (onlyBlock)		// bloky
					{
						blockColl=false;								// postupne zjistuju vsechny souradnice stran zda se prekryvaji
						if (pom[in0]->getOneAAAB(X+min) < pom[in1]->getOneAABB(X+max))						// x
							if (pom[in0]->getOneAAAB(X+max) > pom[in1]->getOneAABB(X+min))					
								if (pom[in0]->getOneAAAB(Y+min) < pom[in1]->getOneAABB(Y+max))				// y
									if (pom[in0]->getOneAAAB(Y+max) > pom[in1]->getOneAABB(Y+min))
										if (pom[in0]->getOneAAAB(Z+min) < pom[in1]->getOneAABB(Z+max))		// z
											if (pom[in0]->getOneAAAB(Z+max) > pom[in1]->getOneAABB(Z+min))
												blockColl=true;			// teleso se nachazi v podprostoru
						if (blockColl&&!Camera)									// kdyz se nachazi v podprostoru
							if (pom[in0]->getNumID()>0)					// za predpokladu ze reference ma nejake ID
								pom[in1]->addID(pom[in0]->getID(0));	// priradim stejne ID do telesa
					}
					if (SphereBlock)	// kole a blok
					{
						// pro kouli a blok musim provest jak kontrolu obalovych kouli tak kontrolu bloku
						sphereColl=false;
						distance=pom[0]->getPos()-pom[1]->getPos();		// spocitam jejich vyajemnou vydalenost
						lenght=distance[0]*distance[0]+distance[1]*distance[1]+distance[2]*distance[2];
																		// pokud je vetsi nez polomer reference a druhe obalove koule
						if (lenght < ((pom[in0]->getActionRadius()+pom[in1]->getSphereRadius())*
									  (pom[in0]->getActionRadius()+pom[in1]->getSphereRadius()))) 
									  sphereColl=true;					// teleso se nachazi v podprostoru reference
						blockColl=false;								// postupne zjistuju vsechny souradnice stran zda se prekryvaji
						if (pom[in0]->getOneAAAB(X+min) < pom[in1]->getOneAABB(X+max))						// x
							if (pom[in0]->getOneAAAB(X+max) > pom[in1]->getOneAABB(X+min))					
								if (pom[in0]->getOneAAAB(Y+min) < pom[in1]->getOneAABB(Y+max))				// y
									if (pom[in0]->getOneAAAB(Y+max) > pom[in1]->getOneAABB(Y+min))
										if (pom[in0]->getOneAAAB(Z+min) < pom[in1]->getOneAABB(Z+max))		// z
											if (pom[in0]->getOneAAAB(Z+max) > pom[in1]->getOneAABB(Z+min))
												blockColl=true;			// teleso se nachazi v podprostoru
						if (blockColl && sphereColl&&!Camera)					// kdyz se nachazi v podprostoru obou teles
							if (pom[in0]->getNumID()>0)					// za predpokladu ze reference ma nejake ID
								pom[in1]->addID(pom[in0]->getID(0));	// priradim stejne ID do telesa
					}
				}else													// jestlize jsou referencni oba dva
				{	// urceni kolize mezi BoundingBoxi referenci
					// zjistim typ BoundingBoxu
					in0=0;
					in1=1;
					if (onlySphere)	// koule
					{
						sphereColl=false;
						distance=pom[0]->getPos()-pom[1]->getPos();		// spocitam jejich vyajemnou vydalenost
						lenght=distance[0]*distance[0]+distance[1]*distance[1]+distance[2]*distance[2];
																		// pokud je vetsi nez polomer reference a druhe obalove koule
						if (lenght < ((pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius())*
									(pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius()))) 
									sphereColl=true;					// teleso se nachazi v podprostoru reference
						if (sphereColl)	// kdyz se srazili obalove telesa
						{
							if (CollSphere)
							{
								this->CollidingSgos.resize(CollidingSgos.size()+1);
								this->CollidingSgos.back().Index[0]=index1;
								this->CollidingSgos.back().Index[1]=i;
								this->CollidingSgos.back().Collision=true;
								this->CollidingSgos.back().Type=this->BALL;
							}
							if (CollBlock || CollSphereBlock) checkTriangleCollision(index1,i);
						}
					}
					if (onlyBlock)		// bloky
					{
						blockColl=false;								// postupne zjistuju vsechny souradnice stran zda se prekryvaji
						if (pom[in0]->getOneAABB(X+min) < pom[in1]->getOneAABB(X+max))						// x
							if (pom[in0]->getOneAABB(X+max) > pom[in1]->getOneAABB(X+min))					
								if (pom[in0]->getOneAABB(Y+min) < pom[in1]->getOneAABB(Y+max))				// y
									if (pom[in0]->getOneAABB(Y+max) > pom[in1]->getOneAABB(Y+min))
										if (pom[in0]->getOneAABB(Z+min) < pom[in1]->getOneAABB(Z+max))		// z
											if (pom[in0]->getOneAABB(Z+max) > pom[in1]->getOneAABB(Z+min))
												blockColl=true;			// teleso se nachazi v podprostoru
						if (blockColl)									// kdyz se srazili obalove telesa
						{
							if (CollSphere)
							{
								this->CollidingSgos.resize(CollidingSgos.size()+1);
								this->CollidingSgos.back().Index[0]=index1;
								this->CollidingSgos.back().Index[1]=i;
								this->CollidingSgos.back().Collision=true;
								this->CollidingSgos.back().Type=this->BALL;
							}
							if (CollBlock || CollSphereBlock) checkTriangleCollision(index1,i);
						}
					}
					if (SphereBlock)	// koule a blok
					{
						// pro kouli a blok musim provest jak kontrolu obalovych kouli tak kontrolu bloku
						sphereColl=false;
						distance=pom[0]->getPos()-pom[1]->getPos();		// spocitam jejich vyajemnou vydalenost
						lenght=distance[0]*distance[0]+distance[1]*distance[1]+distance[2]*distance[2];
																		// pokud je vetsi nez polomer reference a druhe obalove koule
						if (lenght < ((pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius())*
									(pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius()))) 
									sphereColl=true;					// teleso se nachazi v podprostoru reference
						blockColl=false;								// postupne zjistuju vsechny souradnice stran zda se prekryvaji
						if (pom[in0]->getOneAABB(X+min) < pom[in1]->getOneAABB(X+max))						// x
							if (pom[in0]->getOneAABB(X+max) > pom[in1]->getOneAABB(X+min))					
								if (pom[in0]->getOneAABB(Y+min) < pom[in1]->getOneAABB(Y+max))				// y
									if (pom[in0]->getOneAABB(Y+max) > pom[in1]->getOneAABB(Y+min))
										if (pom[in0]->getOneAABB(Z+min) < pom[in1]->getOneAABB(Z+max))		// z
											if (pom[in0]->getOneAABB(Z+max) > pom[in1]->getOneAABB(Z+min))
												blockColl=true;			// teleso se nachazi v podprostoru
						if (blockColl && sphereColl)					// kdyz se srazili obalove telesa
						{
							if (CollSphere)
							{
								this->CollidingSgos.resize(CollidingSgos.size()+1);
								this->CollidingSgos.back().Index[0]=index1;
								this->CollidingSgos.back().Index[1]=i;
								this->CollidingSgos.back().Collision=true;
								this->CollidingSgos.back().Type=this->BALL;
							}
							if (CollBlock || CollSphereBlock) checkTriangleCollision(index1,i);						}
					}
				}
			}
			// urceni kolizi mezi BoundingBoxi
			if (checkID(pom[0],pom[1]))						// jen mezi telesy ve stejnem referencnim prostoru
			{
				if (onlySphere)	// koule
				{
					sphereColl=false;
					distance=pom[0]->getPos()-pom[1]->getPos();		// spocitam jejich vyajemnou vydalenost
					lenght=distance[0]*distance[0]+distance[1]*distance[1]+distance[2]*distance[2];
																	// pokud je vetsi nez polomer reference a druhe obalove koule
					if (lenght < ((pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius())*
								(pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius()))) 
								sphereColl=true;					// teleso se nachazi v podprostoru reference
					if (sphereColl)									// kdyz se srazili obalove telesa
					{
						if (CollSphere)
						{
							this->CollidingSgos.resize(CollidingSgos.size()+1);
							this->CollidingSgos.back().Index[0]=index1;
							this->CollidingSgos.back().Index[1]=i;
							this->CollidingSgos.back().Collision=true;
							this->CollidingSgos.back().Type=this->BALL;
						}
						if (CollBlock || CollSphereBlock) checkTriangleCollision(index1,i);
					}
				}
				if (onlyBlock)		// bloky
				{
					blockColl=false;								// postupne zjistuju vsechny souradnice stran zda se prekryvaji
					if (pom[in0]->getOneAABB(X+min) < pom[in1]->getOneAABB(X+max))						// x
						if (pom[in0]->getOneAABB(X+max) > pom[in1]->getOneAABB(X+min))					
							if (pom[in0]->getOneAABB(Y+min) < pom[in1]->getOneAABB(Y+max))				// y
								if (pom[in0]->getOneAABB(Y+max) > pom[in1]->getOneAABB(Y+min))
									if (pom[in0]->getOneAABB(Z+min) < pom[in1]->getOneAABB(Z+max))		// z
										if (pom[in0]->getOneAABB(Z+max) > pom[in1]->getOneAABB(Z+min))
											blockColl=true;			// teleso se nachazi v podprostoru
					if (blockColl)									// kdyz se srazili obalove telesa
					{
						if (CollSphere)
						{
							this->CollidingSgos.resize(CollidingSgos.size()+1);
							this->CollidingSgos.back().Index[0]=index1;
							this->CollidingSgos.back().Index[1]=i;
							this->CollidingSgos.back().Collision=true;
							this->CollidingSgos.back().Type=this->BALL;
						}
						if (CollBlock || CollSphereBlock) checkTriangleCollision(index1,i);
					}
				}
				if (SphereBlock)	// koule a blok
				{
					// pro kouli a blok musim provest jak kontrolu obalovych kouli tak kontrolu bloku
					sphereColl=false;
					distance=pom[0]->getPos()-pom[1]->getPos();		// spocitam jejich vyajemnou vydalenost
					lenght=distance[0]*distance[0]+distance[1]*distance[1]+distance[2]*distance[2];
																	// pokud je vetsi nez polomer reference a druhe obalove koule
					if (lenght < ((pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius())*
								(pom[in0]->getSphereRadius()+pom[in1]->getSphereRadius()))) 
								sphereColl=true;					// teleso se nachazi v podprostoru reference
					blockColl=false;								// postupne zjistuju vsechny souradnice stran zda se prekryvaji
					if (pom[in0]->getOneAABB(X+min) < pom[in1]->getOneAABB(X+max))						// x
						if (pom[in0]->getOneAABB(X+max) > pom[in1]->getOneAABB(X+min))					
							if (pom[in0]->getOneAABB(Y+min) < pom[in1]->getOneAABB(Y+max))				// y
								if (pom[in0]->getOneAABB(Y+max) > pom[in1]->getOneAABB(Y+min))
									if (pom[in0]->getOneAABB(Z+min) < pom[in1]->getOneAABB(Z+max))		// z
										if (pom[in0]->getOneAABB(Z+max) > pom[in1]->getOneAABB(Z+min))
											blockColl=true;			// teleso se nachazi v podprostoru
					if (blockColl && sphereColl)					// kdyz se srazili obalove telesa
					{
						if (CollSphere)
						{
							this->CollidingSgos.resize(CollidingSgos.size()+1);
							this->CollidingSgos.back().Index[0]=index1;
							this->CollidingSgos.back().Index[1]=i;
							this->CollidingSgos.back().Collision=true;
							this->CollidingSgos.back().Type=this->BALL;
						}
						if (CollBlock || CollSphereBlock) checkTriangleCollision(index1,i);
					}
				}


			}
		}
		pom.pop_back();									// odebrani druheho objektu z pomocneho pole
	}
}
float SgoManager::computeTr(SbVec3f ot, SbMatrix m, SbVec3f o)
{
	SbVec4f r0,r1,r2,r3;
	r0=m[0];
	r1=m[1];
	r2=m[2];
	r3=m[3];
    SbVec3f pom(ot[0]*r0[0]+ot[1]*r1[0]+ot[2]*r2[0],
				ot[0]*r0[1]+ot[1]*r1[1]+ot[2]*r2[1],
				ot[0]*r0[2]+ot[1]*r1[2]+ot[2]*r2[2]);
	return (pom.dot(o))/2.0f;
}
void SgoManager::resolveCollision(const double time, const double dt)
{
	for (unsigned int i=0; i<this->CollidingSgos.size(); i++)
	{
		//cout<<"Pocet kolizi: "<<this->CollidingSgos.size()<<endl;
		bool oneStatic=false;
		int first=this->CollidingSgos[i].Index[0];
		int second=this->CollidingSgos[i].Index[1];
		SgObject * o1=this->sgoList[first];
		SgObject * o2=this->sgoList[second];
		if(o1->getStatic() || o2->getStatic())				// pokud je aspon jeden staticky
		{
			oneStatic=true;
			if (!o1->getStatic())							// tak staticky objekt je ten prvni
			{
				first=this->CollidingSgos[i].Index[1];
				second=this->CollidingSgos[i].Index[0];
			}
		}
		o1=this->sgoList[first];
		o2=this->sgoList[second];

		if ((o1->getType()==SgObject::MISSILE)||(o2->getType()==SgObject::MISSILE))
		{	// zjistim zda se nejedna o strelu
			this->ExplosionMissile(i,time,dt,oneStatic);
			break;
		}
		if (this->CollidingSgos[i].Type==SgoManager::BALL)
		{
			this->resolveCollisionBALL(first,second,oneStatic);
		}
		if (this->CollidingSgos[i].Type==SgoManager::BODY)
		{
			this->resolveCollisionBODY(i,first,second,oneStatic);
		}
		this->sgoList[first]->timeStep(time,dt);
		this->sgoList[second]->timeStep(time,dt);
	}
}
void SgoManager::resolveCollisionBALL(int index1, int index2, bool stat){
	cout<<"Colision BALL"<<endl;
	SgObject * o1=this->sgoList[index1];
	SgObject * o2=this->sgoList[index2];
	SbVec3f axis=o2->getPos()-o1->getPos();
	if (axis.length()>0) axis.normalize();
	else cout<<"Nelze normalizovat CollBall"<<endl;
	SbVec3f U1=o1->getSpeed();
	SbVec3f U2=o2->getSpeed();
	if (stat)
	{
		float mag=U2.length();
		if (mag>0) U2.normalize();
		else cout<<"Nelze normalizovat CollBall"<<endl;
		SbVec3f newSpeed=2*(-U2.dot(axis)) * axis + U2;
		if (newSpeed.length()>0) newSpeed.normalize();
		else cout<<"Nelze normalizovat CollBall"<<endl;
		newSpeed*=mag;
		this->sgoList[index2]->setSpeed(newSpeed);
	}else
	{
		SbVec3f U1x,U1y,U2x,U2y, V1x,V1y,V2x,V2y;
		float M1=o1->getMass();
		float M2=o2->getMass();
		U1x=axis*axis.dot(U1);
		U1y=U1-U1x;
		U2x=-axis*-axis.dot(U2);
		U2y=U2-U2x;
		V1x=((U1x*M1)+(U2x*M2)-(U1x-U2x)*M2)/(M1+M2);
		V2x=((U1x*M1)+(U2x*M2)-(U2x-U1x)*M1)/(M1+M2);
		V1y = U1y;
		V2y = U2y;
		this->sgoList[index1]->setSpeed(V1x+V1y);
		this->sgoList[index2]->setSpeed(V2x+V2y);
		sgoList[index1]->setOldPos();
		sgoList[index1]->setOldOrientation();
	}
    sgoList[index2]->setOldPos();
    sgoList[index2]->setOldOrientation();
}

void SgoManager::resolveCollisionBODY(int index, int index1, int index2, bool stat){
	
	SbVec3f a,b;
	a=sgoList[index1]->getVel();
	b=sgoList[index2]->getVel();
	cout<<a[0]<<" "<<a[1]<<" "<<a[2]<<endl;
	cout<<b[0]<<" "<<b[1]<<" "<<b[2]<<endl;
	this->workIntersectionPoints(index);
	this->getIntersectionPoint(index);
	this->workIntersectionVertexs(index);
	//cout<<"Pocet troj. 0: "<<CollidingSgos[index].Triangles[0].size()/3<<endl;
	//cout<<"Pocet troj. 1: "<<CollidingSgos[index].Triangles[1].size()/3<<endl;
	this->computeNormal(index, stat);

	SbVec3f Impulse(0,0,0);
	double ImpulseNumerator=0;
	double ImpulseDenominator=0;
	SbVec3f vRel(0,0,0);
	float coeficientOfRestitution=1.0f;
	float mass2=this->sgoList[index2]->getMass();
	float OneOverMass=1.0f/(mass2);
	// nacteni pomocnych velicin
	SbVec3f Point=this->CollidingSgos[index].collidingPoint;
	SbVec3f Normal=this->CollidingSgos[index].collidingNormal;
	
	SbVec3f R2=Point-this->sgoList[index2]->getPos();				// rameno (teziste x bod srazky)
	SbMatrix I2=this->sgoList[index2]->getInertiaTensor();			// Inertia tensor

	SbVec3f v2=this->sgoList[index2]->getVel();						// rychlost objektu
	SbVec3f omega2=this->sgoList[index2]->getAngVel();				// uhlova rychlost
	SbVec3f v2omega=this->sgoList[index2]->getAngMom();				// uhlovy moment

	SbVec3f vRel2=v2+omega2.cross(R2);								// relativni rychlost telesa
	SbVec3f R2crossN=R2.cross(Normal);								// R2 x CollisionNormal

	if ((R2crossN[0]+R2crossN[1]+R2crossN[2])==0)
	{
		resolveCollisionBALL(index1,index2,stat);
		return;
	}
	//vypis(&I2);


	//cout<<"Bod srazky   : "<<Point[0]<<" "<<Point[1]<<" "<<Point[2]<<endl;
	//SbVec3f rr = this->sgoList[index2]->getPos();
	//cout<<"Stred telesa : "<<rr[0]<<" "<<rr[1]<<" "<<rr[2]<<endl;
	//cout<<"Rameno       : "<<R2[0]<<" "<<R2[1]<<" "<<R2[2]<<endl;
	//cout<<"V rel        : "<<vRel2[0]<<" "<<vRel2[1]<<" "<<vRel2[2]<<endl;
	//cout<<"R2crossN     : "<<R2crossN[0]<<" "<<R2crossN[1]<<" "<<R2crossN[2]<<" "<<endl;

	I2.multMatrixVec(R2crossN,R2crossN);							// InertiaTensor * (R2xN)
	SbVec3f pom2=R2crossN.cross(R2);								// (IT * (R2xN)) x R2
	if (stat)	// jestlize je jeden staticky (vzdy ten prvni)
	{
		vRel=vRel2;
        ImpulseNumerator = -(1+coeficientOfRestitution)*vRel.dot(Normal);
		ImpulseDenominator = OneOverMass + pom2.dot(Normal);
		if (ImpulseDenominator>0) Impulse = (float)(ImpulseNumerator/ImpulseDenominator) * Normal;
		else
		{
			Impulse=SbVec3f(0,0,0);
			cout<<"Rovnovaha sil"<<endl;
		}
	}else
	{
		SbVec3f R1=Point-this->sgoList[index1]->getPos();
		float mass1=this->sgoList[index1]->getMass();
		OneOverMass +=1.0f/(mass1);
		SbMatrix I1=this->sgoList[index1]->getInertiaTensor();
		SbVec3f v1=this->sgoList[index1]->getVel();
		SbVec3f omega1=this->sgoList[index1]->getAngVel();
		SbVec3f v1omega=this->sgoList[index1]->getAngMom();
		SbVec3f vRel1=v1+omega1.cross(R1);
		vRel=vRel1-vRel2;
		ImpulseNumerator=-(1+coeficientOfRestitution)*vRel.dot(Normal);
		SbVec3f R1crossN=R1.cross(Normal);
		I1.multMatrixVec(R1crossN,R1crossN);
		SbVec3f pom1=R1crossN.cross(R1);
		ImpulseDenominator = OneOverMass + ((pom1+pom2).dot(Normal));
		if (ImpulseDenominator>0) Impulse = (float)(ImpulseNumerator/ImpulseDenominator) * Normal;
		else
		{
			Impulse=SbVec3f(0,0,0);
			cout<<"Rovnovaha sil"<<endl;
		}
		this->sgoList[index1]->setVel(v1+Impulse/mass1);
		SbVec3f pomAM1=v1omega+R1.cross(Impulse);
		this->sgoList[index1]->setAngMom(pomAM1);
		I1.multMatrixVec(pomAM1,pomAM1);
		this->sgoList[index1]->setAngVel(pomAM1);
		sgoList[index1]->setOldPos();
		sgoList[index1]->setOldOrientation();
		Impulse*=-1;
	}

	//cout<<"R2crossN     : "<<R2crossN[0]<<" "<<R2crossN[1]<<" "<<R2crossN[2]<<" "<<endl;
	//cout<<"Pom          : "<<pom2[0]<<" "<<pom2[1]<<" "<<pom2[2]<<" "<<endl;

	//cout<<"Numerator   "<<ImpulseNumerator<<endl;
	//cout<<"Denominator "<<ImpulseDenominator<<endl;
	//cout<<"Impulse      : "<<Impulse[0]<<" "<<Impulse[1]<<" "<<Impulse[2]<<endl;
	//cout<<"Velikost Imp : "<<Impulse.length()<<endl;

	//vypis(&I2);

		SbVec3f v2new;
		v2new = v2 + Impulse/mass2;
		float Eold2=0.5f*mass2*v2.length()*v2.length();
		float Enew2=0.5f*mass2*v2new.length()*v2new.length();
		//cout<<"Energie translace old "<<Eold2<<endl;
		//cout<<"Energie translace new "<<Enew2<<endl;
		cout<<"v old "<<v2.length()<<endl;
		cout<<"v new "<<v2new.length()<<endl;
	this->sgoList[index2]->setVel(v2+Impulse/mass2);
			//vypis(&I2);
			//E+=0.5f*omega2.length()*omega2.length();
//			cout<<"Energie pred "<<E<<endl;
		SbVec3f omegaNew;
	SbVec3f pomAM2=v2omega+R2.cross(Impulse);

	//cout<<"moment old 2 "<<v2omega.length()<<endl;
	//cout<<"moment new 2 "<<pomAM2.length()<<endl;

	this->sgoList[index2]->setAngMom(pomAM2);
	I2.multMatrixVec(pomAM2,pomAM2);
	omegaNew=pomAM2;
	this->sgoList[index2]->setAngVel(pomAM2);
		float tr1,tr2;
		tr1=computeTr(omega2,I2,omega2);
		tr2=computeTr(omegaNew,I2,omegaNew);
		//cout<<"Energie rotace old "<<tr1<<endl;
		//cout<<"Energie rotace new "<<tr2<<endl;
		Eold2+=tr1;
		Enew2+=tr2;
		cout<<"Energie old "<<Eold2<<endl;
		//cout<<"Energie new "<<Enew2<<endl;
		//cout<<"angVel old "<<omega2.length()<<endl;
		//cout<<"angVel new "<<pomAM2.length()<<endl;


		static kolize=0;
		kolize++;
		cout<<"Kolize: "<<kolize;
		//cout<<", Determinant: "<<I2.det3();
		if (Enew2 > 5000) cout<<", Prekrocena energie";
		cout<<endl;


		//cout<<"AngVel    "<<pomAM2[0]<<" "<<pomAM2[1]<<" "<<pomAM2[2]<<endl;

		//newE+=0.5f*pomAM2.length()*pomAM2.length();
			//cout<<"Energie po   "<<newE<<endl;
		//pomAM2=pomAM2/150.f;
			//newE+=0.5f*pomAM2.length()*pomAM2.length();
	//		cout<<"Energie po   "<<newE<<endl;
		//cout<<"AngMoment "<<pomAM2[0]<<" "<<pomAM2[1]<<" "<<pomAM2[2]<<endl;
			//newE+=0.5f*pomAM2.length()*pomAM2.length();
			//cout<<"Energie po   "<<newE<<endl;
//		cout<<"AngVel    "<<pomAM2[0]<<" "<<pomAM2[1]<<" "<<pomAM2[2]<<endl;
	a=sgoList[index2]->getPos();
	c(a);
	sgoList[index2]->setOldPos();
//	sgoList[index2]->setPos(0,0,0);
	sgoList[index2]->setOldOrientation();
	a=sgoList[index2]->getPos();
	c(a);
	//	SbVec3f a,b;
	a=sgoList[index1]->getVel();
	b=sgoList[index2]->getVel();
	cout<<a[0]<<" "<<a[1]<<" "<<a[2]<<endl;
	cout<<b[0]<<" "<<b[1]<<" "<<b[2]<<endl;
	cout<<"Konec kolize"<<endl<<endl;
}


void SgoManager::ExplosionMissile(int index, const double time, const double dt, bool stat)
{
	int first=this->CollidingSgos[index].Index[0];
	int second=this->CollidingSgos[index].Index[1];
	SgObject * o1;
	SgObject * o2;
	bool onlyMissile=false;
	if (!(this->sgoList[second]->getType()==SgObject::MISSILE))	// jako druhy objekt dam strelu 
	{															// (prvni bude static pokud je stat=true)
		first=this->CollidingSgos[index].Index[1];
		second=this->CollidingSgos[index].Index[0];
	}
	o1=this->sgoList[first];
	o2=this->sgoList[second];
	cout<<"Missile: ";
	if (o1->getType()==SgObject::CAMERA) onlyMissile=true;
	if (o1->getType()==SgObject::OBJECT) onlyMissile=true;
	SbVec3f expPos(0,0,0);
	float expDim=0;
	if (onlyMissile)												// pokud vybuchne jen strela
	{
		cout<<"Jen strela ";
		expPos=this->CollidingSgos[index].collidingPoint;
		expPos=o2->getPos();									// pozice pro explozi
		expDim=o2->getSphereRadius()*6;							// rozmeri pro explozi
		if (!stat)												// pokud neni objekt staticky
		{
			SbVec3f dir=o1->getPos()-o2->getPos();					
			if (dir.length()>0)	dir.normalize();					// tak k rychlosti telesa
			else cout<<"Nelze normalizovat ExpMiss"<<endl;
			o1->setVel(o1->getVel()+dir*o2->getSphereRadius());		// prictu silu vybuchu
		}
		this->deleteSgo(second);								// smazu strelu
	}else														// pokud vybuchnou obe telesa
	{
		cout<<"Oba ";
		if (this->CollidingSgos[index].Type==SgoManager::BALL)
		{	// urcim pozici exploze
			expPos=(o2->getPos()-o1->getPos())/2.f;
			expPos+=o2->getPos();
		}else {
			this->workIntersectionPoints(index);
			this->getIntersectionPoint(index);
			expPos=this->CollidingSgos[index].collidingPoint;
		}
		if (expDim < o1->getSphereRadius()*3) expDim=o1->getSphereRadius()*3;	// rozmeri pro explozi
		this->deleteSgo(o1);									// smazu objekt
        this->deleteSgo(o2);									// smazu strelu
		static cam=false;
		if (o1==this->CameraFighter)							// pokud jsem znicil stihac s upnutou kamerou
			this->fighterDestroy=true;
			for (int i=0; i<this->sgoList.getLength(); i++) 
				if (this->sgoList[i]->getType()==SgObject::FIGHTER)	// musim najit prvni volny stihac
				{
					this->CameraFighter=this->sgoList[i];			// a kameru nanej upnout
					cam=true;
					break;
				}
		if (!cam) this->Active=NULL;
		if (!this->free)
		{
			this->setFighterCamera(this->CameraFighter);
			this->setActiveFighter();
		}else this->setActiveFree();
	}
	// exploze
	int k=0;											// najdu prvni volny billboard (k)
	while((k<this->sgoBillboard.getLength())&&(this->sgoBillboard[k]->isUsed())) k++;
	if (k<this->sgoBillboard.getLength()){				// je volny billboard ?
		this->sgoBillboard[k]->setPos(expPos);			// tak nastavim pozici vybuchu
		this->sgoBillboard[k]->setDimension(expDim);	// velikost vybuchu
		this->sgoBillboard[k]->useBillboard();			// a oznacim billboard jako pouzity
		sgoRoot->addChild(this->sgoBillboard[k]->getSceneGraph());	// pridam ho do sceny
	}else{												// neni volny billboard
		Billboard * b = new Billboard;					// tak ho vytvorim
		b->setDimension(expDim);						// nastavim velikost
		b->setTexture("test.tga",4);					// a nactu texturu vybuchu
		b->setTextureFPS(10);							// rychlost promitani text.
		b->setPos(expPos);								// pozici
		b->setCamera(this->Camera);						// ukazatel na kameru
		b->useBillboard();								// bilboard pouziju
		b->setManager(this->sgoRoot);
		b->timeStep(time,dt);						
		sgoRoot->addChild(b->getSceneGraph());			// pridam ho do sceny
		this->sgoBillboard.append(b);
	}
	cout<<" detonated!"<<endl;
}

SoIntersectionDetectionAction::Resp SgoManager::intersectionCb(void * closure,
    const SoIntersectingPrimitive *p1, const SoIntersectingPrimitive *p2)
{
	SgoManager * ThisM = (SgoManager*)closure;		// ukazatel na class manager
	vector<SbVec3f> points;
	vector<SbVec3f > u1,u2,s1,s2;
	vector<float > n2DOTu1, n1DOTu2;
	float t;
	SbVec3f point,l1,l2;
	for (int i=0; i<3; i++)
	{
		u1.push_back(p1->xf_vertex[(i+1)%3]-p1->xf_vertex[i%3]);		// usecka v troj. p1
		s1.push_back((p1->xf_vertex[(i+1)%3]+p1->xf_vertex[i%3])/2.f);	// stred usecky
		u2.push_back(p2->xf_vertex[(i+1)%3]-p2->xf_vertex[i%3]);		// usecka v troj. p2
		s2.push_back((p2->xf_vertex[(i+1)%3]+p2->xf_vertex[i%3])/2.f);	// stred usecky
	}
	SbVec3f n1=u1[0].cross(u1[1]);								// normala pro troj. p1
	n1.normalize();
	SbVec3f n2=u2[0].cross(u2[1]);								// normala pro troj. p2
	n2.normalize();

	//cout<<"*"<<endl;
	ThisM->CollidingSgos.back().Collision=true;

	// urceni vzajemne polohy roviny troj a primky //n.a=0 => rovnobezne
	for (int i=0; i<3; i++)
	{
		n2DOTu1.push_back(n2.dot(u1[i]));
		n1DOTu2.push_back(n1.dot(u2[i]));
		if (n2DOTu1[i]!=0)
		{													// ruznobezne
			t=(n2.dot(p2->xf_vertex[0]-p1->xf_vertex[i%3]))/n2DOTu1[i];
			point=p1->xf_vertex[i%3]+u1[i]*t;
			l1=(s1[i]-p1->xf_vertex[i%3]);					// vzdalenost krajniho bodu ke stredu musi byt vetsi nez
			l2=(s1[i]-point);								// vzdalenost pruseciku ke stredu
			if (l1.length()>l2.length())
			{												// zjistim zda prusecik je soucasti druheho troj.
//				cout<<"na usecce ";
				if (partOfTriangle(p2->xf_vertex[0]-point,p2->xf_vertex[1]-point,p2->xf_vertex[2]-point))
				{
					points.push_back(point);				// pridam bod do seznamu platnych pruseciku
//					cout<<"v troj. ";
				}
			}
		}else
		{													// rovnobezne
			cout<<"rovnobezny ";
			if (n2.dot(p2->xf_vertex[0]-p1->xf_vertex[i%3])==0)
			{												// primka je soucasti roviny
				point=s1[i];								// prusecik je stred usecky
				if (partOfTriangle(p2->xf_vertex[0]-point,p2->xf_vertex[1]-point,p2->xf_vertex[2]-point))
					points.push_back(point);				// pridam bod do seznamu platnych pruseciku
			}
		}
		//cout<<point[0]<<" "<<point[1]<<" "<<point[2]<<endl;
		if (n1DOTu2[i]!=0)
		{													// ruznobezne
			t=(n1.dot(p1->xf_vertex[0]-p2->xf_vertex[i%3]))/n1DOTu2[i];
			point=p2->xf_vertex[i%3]+u2[i]*t;
			l1=(s2[i]-p2->xf_vertex[i%3]);					// vzdalenost krajniho bodu ke stredu musi byt vetsi nez
			l2=(s2[i]-point);								// vzdalenost pruseciku ke stredu
			if (l1.length()>l2.length())
			{												// zjistim zda prusecik je soucasti druheho troj.
				//cout<<"na usecce ";
				if (partOfTriangle(p1->xf_vertex[0]-point,p1->xf_vertex[1]-point,p1->xf_vertex[2]-point))
				{
					points.push_back(point);				// pridam bod do seznamu platnych pruseciku
				//	cout<<"v troj. ";
				}
			}
		}else
		{													// rovnobezne
			cout<<"rovnobezny ";
			if (n1.dot(p1->xf_vertex[0]-p2->xf_vertex[i%3])==0)
			{												// primka je soucasti roviny
				point=s2[i];								// prusecik je stred usecky
				if (partOfTriangle(p1->xf_vertex[0]-point,p1->xf_vertex[1]-point,p1->xf_vertex[2]-point))
					points.push_back(point);				// pridam bod do seznamu platnych pruseciku
			}
		}
		//cout<<point[0]<<" "<<point[1]<<" "<<point[2]<<endl;
	}

	while (points.size()){ // prirazeni vsech koliznich bodu do vektoru
		ThisM->CollidingSgos.back().Points.push_back(points.back());
		points.pop_back();
	}
	if(ThisM->CollidingSgos.back().Points.size()<1) ThisM->CollidingSgos.back().Points.push_back(p1->xf_vertex[1]);

	if (!p1->path->getIndex(1)){
		ThisM->CollidingSgos.back().Triangles[0].push_back(p1->xf_vertex[0]);
		ThisM->CollidingSgos.back().Triangles[0].push_back(p1->xf_vertex[1]);
		ThisM->CollidingSgos.back().Triangles[0].push_back(p1->xf_vertex[2]);
	}else{
		ThisM->CollidingSgos.back().Triangles[1].push_back(p1->xf_vertex[0]);
		ThisM->CollidingSgos.back().Triangles[1].push_back(p1->xf_vertex[1]);
		ThisM->CollidingSgos.back().Triangles[1].push_back(p1->xf_vertex[2]);
	}
	if (!p2->path->getIndex(1)){
		ThisM->CollidingSgos.back().Triangles[0].push_back(p2->xf_vertex[0]);
		ThisM->CollidingSgos.back().Triangles[0].push_back(p2->xf_vertex[1]);
		ThisM->CollidingSgos.back().Triangles[0].push_back(p2->xf_vertex[2]);
	}else{
		ThisM->CollidingSgos.back().Triangles[1].push_back(p2->xf_vertex[0]);
		ThisM->CollidingSgos.back().Triangles[1].push_back(p2->xf_vertex[1]);
		ThisM->CollidingSgos.back().Triangles[1].push_back(p2->xf_vertex[2]);
	}
	return SoIntersectionDetectionAction::NEXT_PRIMITIVE;
}
bool SgoManager::partOfTriangle(SbVec3f a, SbVec3f b, SbVec3f c)
{
	double th=0;
	if (a.length()>0) a.normalize();
	if (b.length()>0) b.normalize();
	if (c.length()>0) c.normalize();
	th+=acos(a.dot(b));
	th+=acos(b.dot(c));
	th+=acos(c.dot(a));
	// 2*PI=6.283184
	if ((th>6.28317)&&(th<6.28319)) return true;
	else return false;
}
float SgoManager::diffExp2(SbVec3f v1, SbVec3f v2){
  SbVec3f pom=v1-v2;
  return pom[0]*pom[0]+pom[1]*pom[1]+pom[2]*pom[2];
}
void SgoManager::workIntersectionPoints(int index){
	vector<SbVec3f> pomP;
	cout<<"work ";
    cout<<CollidingSgos[index].Points.size()<<endl;
	for (unsigned int i=0;i<CollidingSgos[index].Points.size();i++){
		cout<<"in";
		bool dual=false;
		for (unsigned int j=i+1;j<CollidingSgos[index].Points.size();j++)
			if (diffExp2(CollidingSgos[index].Points[i],CollidingSgos[index].Points[j])<this->epsilon) dual=true;
		if (dual) pomP.push_back(CollidingSgos[index].Points[i]);
	}
	while (CollidingSgos[index].Points.size()) CollidingSgos[index].Points.pop_back();
	for (unsigned int i=0;i<pomP.size();i++) 
	{
		//cout<<"Point "<<pomP[i][0]<<" "<<pomP[i][1]<<" "<<pomP[i][2]<<" "<<endl;	
		CollidingSgos[index].Points.push_back(pomP[i]);
	}
	if (pomP.size()==0) CollidingSgos[index].Points.push_back(CollidingSgos[index].Points[0]);
}
void SgoManager::workIntersectionVertexs(int index){ //eliminuje troj.
	vector<SbVec3f> pomP;
	bool a,b,c;
	for (int k=0;k<2;k++){
		pomP.resize(0);
		for(unsigned int i=0;i<this->CollidingSgos[index].Triangles[k].size();i+=3){
			for (unsigned int j=i+3;j<this->CollidingSgos[index].Triangles[k].size();j+=3){
				a=b=c=false;
				for (int l=0;l<3;l++){
					if (this->CollidingSgos[index].Triangles[k][i+0]==this->CollidingSgos[index].Triangles[k][j+l]) a=true;
					if (this->CollidingSgos[index].Triangles[k][i+1]==this->CollidingSgos[index].Triangles[k][j+l]) b=true;
					if (this->CollidingSgos[index].Triangles[k][i+2]==this->CollidingSgos[index].Triangles[k][j+l]) c=true;
				}
				if (a&&b&&c) {
					this->CollidingSgos[index].Triangles[k][j+0]=SbVec3f(-999,-999,-999);
					this->CollidingSgos[index].Triangles[k][j+1]=SbVec3f(-999,-999,-999);
					this->CollidingSgos[index].Triangles[k][j+2]=SbVec3f(-999,-999,-999);
				}
			}
		}
		for(unsigned int i=0;i<this->CollidingSgos[index].Triangles[k].size();i++)
			if (this->CollidingSgos[index].Triangles[k][i][0]!=-999) pomP.push_back(this->CollidingSgos[index].Triangles[k][i]);
		while (this->CollidingSgos[index].Triangles[k].size()) this->CollidingSgos[index].Triangles[k].pop_back();
		for(unsigned int i=0;i<pomP.size();i++)this->CollidingSgos[index].Triangles[k].push_back(pomP[i]);
	}
	//cout<<"Prvni objekt, pocet troj. "<<(unsigned int)this->CollidingSgos[index].Triangles[0].size()/3<<endl;
	//cout<<"Druhy objekt, pocet troj. "<<(unsigned int)this->CollidingSgos[index].Triangles[1].size()/3<<endl;
	//	cout<<"pole "<<endl;
//	for(int i=0;i<index.size();i++)	cout<<index[i]<<endl;
}

SbVec3f SgoManager::getIntersectionPoint(int index){
  SbVec3f CP(0,0,0);
  for (unsigned int i=0;i<CollidingSgos[index].Points.size();i++) CP+=CollidingSgos[index].Points[i];
  CP/=(const float)CollidingSgos[index].Points.size();
  this->CollidingSgos[index].collidingPoint=CP;
  return CP;
}
void SgoManager::computeNormal(int index, bool stat){
	int indexTriangle=0;
	SbVec3f normala(0,0,0);
	//cout<<"pocet tr. 1 telesa "<<this->CollidingSgos[index].Triangles[0].size()<<endl;
	//cout<<"pocet tr. 2 telesa "<<this->CollidingSgos[index].Triangles[1].size()<<endl;
	if (this->CollidingSgos[index].Triangles[0].size()>this->CollidingSgos[index].Triangles[1].size()) 
		indexTriangle=1;
	if (this->sgoList[(this->CollidingSgos[index].Index[0])]->getStatic())
		indexTriangle=0;
	if (this->sgoList[(this->CollidingSgos[index].Index[1])]->getStatic())
		indexTriangle=1;

	//cout<<this->CollidingSgos[index].Triangles[indexTriangle].size()<<endl;
	for (unsigned int i=0;i<this->CollidingSgos[index].Triangles[indexTriangle].size()-1;i+=3)
	{
		SbVec3f a=this->CollidingSgos[index].Triangles[indexTriangle][i]-this->CollidingSgos[index].Triangles[indexTriangle][i+1];
		SbVec3f b=this->CollidingSgos[index].Triangles[indexTriangle][i+1]-this->CollidingSgos[index].Triangles[indexTriangle][i+2];
		SbVec3f pom=a.cross(b);
		if (pom.length()>0) pom.normalize();
		else cout<<"Nelze normalizovat computeNormal"<<endl;
		normala+=pom;
	}
	if (normala.length()>0) normala.normalize();
	else cout<<"Nelze normalizovat computeNormal"<<endl;
	if (indexTriangle) normala*=-1;
	this->CollidingSgos[index].collidingNormal=normala;
	//cout<<"Normala x: "<<normala[0]<<" y: "<<normala[1]<<" z: "<<normala[2]<<endl;
}
void SgoManager::timeStep(const double time, const double dt)
{
	//SbList<SgObject*> tmp = sgoList;
	//int c = tmp.getLength();

	//for (int i=0; i<c; i++)
	//{
	//	// Following condition is necessary, because any sgo can
	//	// remove itself or other sgos from the list during the execution
	//	// of the following call to SgObject::timeStep(time, dt).
	//	if (sgoList.find(tmp[i]) != -1)
	//	{
	//		tmp[i]->setForce(this->Force);
	//		tmp[i]->timeStep(time, dt);
	//	}
	//}
	  
	for (int i=0; i<this->sgoBillboard.getLength(); i++)
	{
		if (this->sgoBillboard[i]->isUsed()) 
			this->sgoBillboard[i]->timeStep(time,dt);
		//else sgoRoot->removeChild(this->sgoBillboard[i]->getSceneGraph());
	}

	this->checkCollision(time,dt);
	//this->resolveCollision(time,dt);
}

	/*for (unsigned int i=0; i<this->CollidingSgos.size(); i++)
	{
		if (this->CollidingSgos[i].Type==SgoManager::BALLvsBALL)
			this->resolveCollisionBALLvsBALL(i);
		if (this->CollidingSgos[i].Type==SgoManager::BODYvsBODY)
			this->resolveCollisionBODYvsBODY(i);
*/
/*		if (this->CollidingSgos[i].Type==SgObject::EXPLOSION)
			this->resolveCollisionExplosion(time,dt,i);
		if (this->CollidingSgos[i].Type==SgObject::BALL)
			this->resolveCollisionSphere(i);
		if (this->CollidingSgos[i].Type==SgObject::BODY)
		{
			//resolve collision
			//	  cout<<"pocet prvniho"<<this->CollidingSgo.firstTriangles.size()/3.f<<endl;
		//	  cout<<"pocet druheho"<<this->CollidingSgo.secondTriangles.size()/3.f<<endl;
		//	  cout<<"pocet koliznich bodu"<<this->CollidingSgos.Points.size()<<endl;
			cout<<"body !!!"<<endl;
			this->workIntersectionPoints(i);
			SbVec3f CM = this->getIntersectionPoint(i);
			this->workIntersectionVertexs(i);
			this->computeNormal(i);
			this->resolveCollisionBody(i);
  			sgoList[this->CollidingSgos[i].Index[0]]->setOldPos();
			sgoList[this->CollidingSgos[i].Index[0]]->setOldOrientation();
			sgoList[this->CollidingSgos[i].Index[1]]->setOldPos();
			sgoList[this->CollidingSgos[i].Index[1]]->setOldOrientation();
			cout<<"konec"<<endl;*/
