#ifndef SGO_MANAGER_H_ZERO
#define SGO_MANAGER_H_ZERO
/************************************************************************

   SgoManager.h - scene objects manager class

   Written by Petr Mastera (xmaste00) and PCJohn (peciva).


   THIS SOFTWARE IS NOT COPYRIGHTED

   This source code is offered for use in the public domain.
   You may use, modify or distribute it freely.

   This source code is distributed in the hope that it will be useful but
   WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
   DISCLAIMED.  This includes but is not limited to warranties of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   If you find the source code useful, authors will kindly welcome
   if you give them credit and keep their names with their source code,
   but you are not forced to do so.

************************************************************************/

#include "MainInclude.h"
#include "SgObject.h"
#include "SgInterface.h"
#include "Billboard.h"

#include "test.h"

class SgObject;
class Billboard;

/**  SgoManager class.
 * SgoManager class manages the set of SgObjects.
 *
 * The word SgoManager is abbreviation of SpaceGame Object Manager.
 */

class SgoManager {
public:
  
	//enum COLLISION_TYPE { BODY&BODY = 0x01, BALL&BALL = 0x02, BODY&BALL = 0x03, EXPLOSION = 0x04 };
	#define G 6.67e-11f
	enum CollidingType { BODY = 0x01, BALL = 0x02 };
	
private:
	SoSeparator * sgoRoot;       
	SbList<SgObject*> sgoList;  
	SbList<Billboard*> sgoBillboard;

	SgObject * Active;
	SgObject * CameraFree;
	SgObject * CameraFighter;

	bool free;
	bool fighterDestroy;
	SoPerspectiveCamera * Camera;
	SoGroup * collisionRoot;    

	SbVec3f Force;					// translacni sili pusobici na teleso
	SbVec3f Torque;					// rotacni sili pusobici na teleso
	float Damp;						// tlumeni

	float epsilon;
	typedef struct 	// uchovava info o kolidujicich objektech
	{
		bool Collision;					// nastala kolize?
		CollidingType Type;				// typ kolize
		int Index[2];						// index kolidujicich objektu
		vector<SbVec3f> Points;			// body
		vector<SbVec3f> Triangles[2];		// trojuhelniky
		SbVec3f collidingPoint;			// bod srazky
		SbVec3f collidingNormal;			// normala srazky
	} Type_CollidingSgos;
	vector <Type_CollidingSgos > CollidingSgos;

	SoIntersectionDetectionAction * ida; 
	static SoIntersectionDetectionAction::Resp intersectionCb(void *closure, const SoIntersectingPrimitive *p1, const SoIntersectingPrimitive *p2);
    static bool partOfTriangle(SbVec3f a, SbVec3f b, SbVec3f c);
	void checkCollision(const double time, const double dt);
	bool checkID(SgObject * sgo1, SgObject * sgo2);
	void checkTriangleCollision(int sgo1, int sgo2);
	void checkCollisionWith(SgObject * sgo, int index);

	float computeTr(SbVec3f ot, SbMatrix m, SbVec3f o);

	void resolveCollision(const double time, const double dt);
	void resolveCollisionBALL(int index1, int index2, bool stat);
	void resolveCollisionBODY(int index, int index1, int index2, bool stat);
	void ExplosionMissile(int index, const double time, const double dt, bool stat);
		
	float diffExp2(SbVec3f v1, SbVec3f v2);
	void workIntersectionPoints(int index);
	void workIntersectionVertexs(int index);
	SbVec3f getIntersectionPoint(int index);
	void computeNormal(int index, bool stat);
		
public:
  SgoManager();
  ~SgoManager();


  void setFreeCamera(SgObject * o) { this->CameraFree=o; }
  void setFighterCamera(SgObject * o) { this->CameraFighter=o; }
  void setActiveFighter() { this->Active=this->CameraFighter; this->free=false; }
  void setActiveFree() { this->Active=this->CameraFree; this->free=true;}
  bool destroy() { return this->fighterDestroy; }
  bool newFighter() { this->fighterDestroy=false; }

  SoSeparator * getSgoRoot() const { return sgoRoot; }

  void setCamera(SoPerspectiveCamera * camera) { this->Camera=camera; }

  SgObject * getSgo(int index)  { return sgoList[index]; }
  const SgObject* getSgo(int index) const  { return sgoList[index]; }
  int getSgosNum() const  { return sgoList.getLength(); }

  SgObject * getActive() { return this->Active; }
	
	void setForce(SbVec3f f) { this->Force=f; }
	SbVec3f getForce() { return this->Force; }
	void setTorque(SbVec3f f) { this->Torque=f; }
	SbVec3f getTorque() { return this->Torque; }
	void setDamp(float d) { this->Damp=d; }
	float getDamp() { return this->Damp; }

  void appendSgo(SgObject *sgo);
  void removeSgo(SgObject *sgo);
  void removeSgo(int index);
  void deleteSgo(int index);
  void deleteSgo(SgObject *sgo);
  void deteleAllSgos();
  
  void timeStep(const double time, const double dt);
};

#endif /* SGO_MANAGER_H */
